<?php

/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\LogView\Controller\Adminhtml\Log;

use Magento\Backend\App\Action\Context;

class ViewAjax extends \Magento\Backend\App\Action {

    protected $jsonResultFactory;
    public $log;
    protected $directory_list;
    private $maxSizeToLoad;

    public function __construct(
    Context $context, \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory, \Magento\Framework\App\Filesystem\DirectoryList $directory_list
    ) {
        parent::__construct($context);
        $this->maxSizeToLoad = 2097152;
        $this->directory_list = $directory_list;
        $this->log = $this->directory_list->getPath('log') . DIRECTORY_SEPARATOR . $this->getRequest()->getParam("file_name");
        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * Index action
     *
     * @return \Magento\Backend\Model\View\Result\Page
     */
    public function execute() {
        if ($this->getRequest()->isXmlHttpRequest()) {
            return $this->getNewLines($this->getRequest()->getParam("lastsize"), $this->getRequest()->getParam("grep"), $this->getRequest()->getParam("invert"));
        }
    }

    /**
     * Is the user allowed to view the store grid.
     *
     * @return bool
     */
    protected function _isAllowed() {
        return $this->_authorization->isAllowed('FiloBlu_LogView::general');
    }

    /**
     * This function is in charge of retrieving the latest lines from the log file
     * @param string $lastFetchedSize The size of the file when we lasted tailed it.
     * @param string $grepKeyword The grep keyword. This will only return rows that contain this word
     * @return Returns the JSON representation of the latest file size and appended lines.
     */
    public function getNewLines($lastFetchedSize, $grepKeyword, $invert) {

        $resultJson = $this->resultJsonFactory->create();

        /**
         * Clear the stat cache to get the latest results
         */
        clearstatcache();
        /**
         * Define how much we should load from the log file
         * @var
         */
        $fsize = filesize($this->log);
        $maxLength = ($fsize - $lastFetchedSize);

        /**
         * Verify that we don't load more data then allowed.
         */
        if ($maxLength > $this->maxSizeToLoad) {
            $resultJson->setData(array("size" => $fsize, "data" => array("ERROR: PHPTail attempted to load more (" . round(($maxLength / 1048576), 2) . "MB) then the maximum size (" . round(($this->maxSizeToLoad / 1048576), 2) . "MB) of bytes into memory. You should lower the defaultUpdateTime to prevent this from happening. ")));
            return $resultJson;
        }
        /**
         * Actually load the data
         */
        $data = array();
        if ($maxLength > 0) {

            $fp = fopen($this->log, 'r');
            fseek($fp, -$maxLength, SEEK_END);
            $data = explode("\n", fread($fp, $maxLength));
        }
        /**
         * Run the grep function to return only the lines we're interested in.
         */
        if ($invert == 0) {
            $data = preg_grep("/$grepKeyword/", $data);
        } else {
            $data = preg_grep("/$grepKeyword/", $data, PREG_GREP_INVERT);
        }
        /**
         * If the last entry in the array is an empty string lets remove it.
         */
        if (end($data) == "") {
            array_pop($data);
        }
        $resultJson->setData(array("size" => $fsize, "data" => $data));

        return $resultJson;
    }

}
